{-----------------------------------------------------------------------------

   Este exemplo de streaming mostra como  a tcnica de enviar e receber
   amostras atravs da aplicao.

   Quando se liga a comunicacao de um canal para o outro E vice-versa, ocorre
   uma degenerao no sinal causadas por fenomenos de eco, etc...

   Para uma utilizao eficaz, o programador provavelmente dever implementar
   sua rotina de cancelamento de eco.

   Dever funcionar apenas na versao >= 2.75 da VoicerLib

-----------------------------------------------------------------------------}

unit fStreaming;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, OleCtrls, VOICERLIB200Lib_TLB, StdCtrls, ComCtrls, LogMemo,
  SyncObjs;

type

  TSampleData = class
    szVal: array[0..160] of smallint;    
  end;


  TForm1 = class(TForm)
    vlb: TVoicerLib200;
    Button1: TButton;
    Button2: TButton;
    Label1: TLabel;
    lblAmostra1: TLabel;
    Button3: TButton;
    Button4: TButton;
    log1: TLogMemo;
    chkSampleToApp: TCheckBox;
    Memo1: TMemo;
    Label2: TLabel;
    chkCH1Fala: TCheckBox;
    chkCh1Envia: TCheckBox;
    chkCH2Fala: TCheckBox;
    chkCH2Envia: TCheckBox;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure vlbRingDetected(ASender: TObject; Port: Smallint);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure vlbSampleReceived(ASender: TObject; Port: Smallint);
    procedure chkSampleToAppClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure chkCH1FalaClick(Sender: TObject);
    procedure chkCh1EnviaClick(Sender: TObject);
    procedure chkCH2EnviaClick(Sender: TObject);
    procedure chkCH2FalaClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }

  end;

  //Thread que pega as amostras e joga para o outro canal
 TWorkerThread = class(TThread)
  private
    Work: TOBject;
    FLastException: Exception;
    FQueue: TThreadlist;
    FWakeupCall: TSimpleEvent;
    procedure ShowException;
  protected
    procedure Execute; override;
    procedure HandleException(E: Exception);
    procedure Initialize;
    procedure Process;
    procedure Uninitialize;
    procedure EnviaAmostras;

  public
    target: short;
    constructor Create;
    destructor Destroy; override;
    procedure Add( aObject: TObject );
    procedure WakeUp;
    procedure MostraFinaliza;
  end;

var
  Form1: TForm1;
//  fGrava: File;
  SamplesRec: integer;
  iSeq, iSeqAnt: array[1..2] of smallint;
  nContaThread: array[1..2] of integer;
  sDirApp: string;

  MyThread: array[1..2] of TWorkerThread;

implementation

{$R *.dfm}

//------------------------------------------------------------------------------
// Thread que pegar as amostras e jogar para o outro canal
//------------------------------------------------------------------------------
procedure TWorkerThread.EnviaAmostras;
begin

end;

procedure TWorkerThread.Add(aObject: TObject);
begin
  Assert( Assigned( aObject ));
  FQueue.Add(aObject);
//  FWakeupCall.SetEvent;
end;

procedure TWorkerThread.WakeUp;
begin
  FWakeupCall.SetEvent;
end;

procedure TWorkerThread.MostraFinaliza;
begin
  Form1.log1.AddLine('Thread terminada');
end;


constructor TWorkerThread.Create;
begin
  inherited Create(true);
  FQueue := TThreadlist.Create;
  FWakeupCall:= TSimpleEvent.Create;
  Resume;
end;

destructor TWorkerThread.Destroy;
var
  I: Integer;
  List: TList;
begin
  Terminate;
  FWakeupCall.SetEvent;
  inherited;
  if Assigned(FQueue) then begin
    List:= FQueue.LockList;
    try
      for i:= 0 to List.Count-1 do
        TObject(List[I]).Free;
    finally
      FQueue.UnlockList;
    end; { finally }
    FQueue.Free;
  end;
  FWakeupCall.Free;
end;
procedure TWorkerThread.Execute;
var
  List: TList;
begin
  try
    Initialize;
    while not Terminated do
    begin
      try
        FwakeupCall.WaitFor(INFINITE);
        if not Terminated then
        begin
          repeat
            List:= FQueue.LockList;
            try
              if List.Count > 0 then begin
                Work := TObject(List[0]);
                List.Delete(0);
              end
              else
                Work := nil;
            finally
              FQueue.UnlockList;
            end; { finally }
            if Assigned(Work) and not Terminated then
              Synchronize(Process);


            sleep(20);
            //FwakeupCall.WaitFor(200);


          until (Work = nil) or Terminated;
        end;
      except
        On E: Exception do
          HandleException(E);
        // for some types of exception you may want to break
        // out of the loop since the thread cannot continue
      end; { except }
      //teste
      if Work = nil then
        break;
    end; {while}
  except
    On E: Exception do
      HandleException(E);
  end; { except }
  
  synchronize(MostraFinaliza);
  Uninitialize; // must not raise exceptions!
end;

procedure TWorkerThread.HandleException(E: Exception);
begin
  FLastException := E;
  Synchronize(ShowException);
end;

procedure TWorkerThread.Initialize;
begin
  // place code to create and configure things in
  // the threads context here.
end;

procedure TWorkerThread.Process;
var
  pArray: OleVariant;
  i: integer;
begin
  // do something with the Work object here. Make sure
  // you either free it at the end, if it is no longer
  // needed, or send it to some other object for further
  // processing, e.g. as a message
  // PostMessage( SomeReceiverWindow, UM_PROCESSINGDONE,
  // Wparam(Work), 0 );

  //cria vetor
  pArray := VarArrayCreate([0, 160], varSmallint);

  //copia com caracter de controle
  for i := 0 to 160 do
    pArray[i] := TSampleData(Work).szVal[i];

  i := Form1.vlb.PutSamples(target,pArray);
  if (i<>0) then
    form1.log1.AddLine('Erro no putsamples - ' + IntToStr(i));

  TSampleData(Work).Free;

end;

procedure TWorkerThread.ShowException;
begin
  Application.HandleException(FLastException);
end;

procedure TWorkerThread.Uninitialize;
begin
  // place code to undo what Initialize did here
end;

//------------------------------------------------------------------------------
// Programa principal
//------------------------------------------------------------------------------
procedure TForm1.Button1Click(Sender: TObject);
var
  ret: integer;
begin;
  ret := vlb.StartVoicerLib;
  if ret <> 99 then
    Log1.AddLine('Erro ao iniciar o driver - codigo ' + IntToStr(ret))
  else
    Log1.AddLine('Placa iniciada com sucesso!');

end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  vlb.ShutdownVoicerLib;
  Log1.AddLine('Placa finalizada!');
end;

procedure TForm1.vlbRingDetected(ASender: TObject; Port: Smallint);
begin
  vlb.PickUp(Port,100);
  log1.AddLine('Porta ' + IntToStr(Port) + ' atendeu');
end;

procedure TForm1.Button3Click(Sender: TObject);
var
  i: integer;
begin
  for i := 1 to vlb.PortsCount do
  begin
    vlb.DisableSampleToApp(i);
    vlb.DisableSampleToCard(i);
    vlb.HangUp(i);
  end;
end;

procedure TForm1.Button4Click(Sender: TObject);
var
  f: File;
  i,nVal: smallint;
  szVal: array[0..159] of smallint;
  pArray: OleVariant;
  timeout,iConta: integer;
  dt: TSampleData;
begin
  if chkSampleToApp.Checked then
  begin
    ShowMessage('Desabilite a recepo antes de iniciar a reproduo! Leia os comentrios abaixo');
  end;

  Log1.AddBoxTitle('*',60,'Inicio');

  //cria working thread para os dois canais
  MyThread[1] := TWorkerThread.Create;
  MyThread[1].target := 2;

  vlb.SetPlayFormat(2,ffWavePCM);
  AssignFile(f,sDirApp + 'teste_grava.raw');
  Log1.AddLine('Abrindo ' + sDirApp +  'teste_grava.raw');
  Screen.Cursor := crHourglass;

  reset(f,1);

   //pula cabealho
  Seek(f,92);

  //Joga silencio
  vlb.EnableSampleToCard(2);

  iConta := 0;
  while (not Eof(f)) do
  begin

      //cria vetor
      pArray := VarArrayCreate([0, 160], varSmallint);

      //le amostra
      try
        BlockRead(f,szVal,320);  //16*sizeof(short)
      except
        break;
      end;

      //copia dados para a thread
      dt := TSampleData.Create;
      dt.szVal[0] := $80;
      for i := 1 to 160 do
          dt.szVal[i] := szVal[i-1];

      MyThread[1].Add(TObject(dt));
      Inc(iConta);
      inc(nContaThread[1]);
      if (nContaThread[1]>20) then
      begin
        nContaThread[1] := 0;
        MyThread[1].WakeUp;
      end;

{
      //copia com caracter de controle
      pArray[0] := $80;
      for i := 1 to 160 do
        pArray[i] := szVal[i-1];

      timeout := 0;
      while (vlb.PutSamples(2,pArray) <> 0) do
      begin
        Sleep(2);
        Inc(Timeout);
        if timeout>20 then
          break;
      end;
}

{      if (iConta = 20) then
      begin
        Log1.AddLine('Habilitou Sample ' + IntToStr(iConta));
        vlb.EnableSampleToCard(2);
      end;

      if (iConta=999) then
        break;}

  end;
  MyThread[1].WakeUp;

  closefile(f);
  Screen.Cursor := crDefault;
  Log1.AddLine('Fim de Arquivo - iConta=' + IntToStr(iConta));

end;

procedure TForm1.vlbSampleReceived(ASender: TObject; Port: Smallint);
var
  pArray: OleVariant;
  szVal: array[0..159] of smallint;
  ret,NumWritten,i,j: integer;
  dt: TSampleData;
begin

    pArray := VarArrayCreate([0, 160], varSmallint);
    pArray := vlb.GetSamples(Port);

    //pega sequencial
    iSeq[Port] := pArray[0];
    if (iSeq[Port] <> (iSeqAnt[Port] + 1)) then
      Log1.AddLine('Erro sequencial -> ' + IntToStr(iSeq[Port]) + ' / '  + IntToStr(iSeqAnt[Port]));

{   for i := 0 to 160 do
        szVal[i] := pArray[i];

    BlockWrite(fGrava,szVal,320,NumWritten);
    if (NumWritten=0) then
        NumWritten := 1;}

    //copia dados para a thread
    dt := TSampleData.Create;
    dt.szVal[0] := $80;
    for i := 1 to 160 do
        dt.szVal[i] := pArray[i];

    MyThread[Port].Add(TObject(dt));

    inc(nContaThread[Port]);
    if (nContaThread[Port]>20) then
    begin
      nContaThread[Port] := 0;
      MyThread[Port].WakeUp;
    end;


    Inc(SamplesRec);
    lblAmostra1.Caption := IntToStr(Port);

    iSeqAnt[Port] := iSeq[Port];
end;

procedure TForm1.chkSampleToAppClick(Sender: TObject);
var
  i:integer;
begin
  if  chkSampleToApp.Checked then
  begin

    //cria working threads
    MyThread[1] := TWorkerThread.Create;
    MyThread[1].target := 2;    //o target  a porta de destino

    MyThread[2] := TWorkerThread.Create;
    MyThread[2].target := 1;    //o target  a porta de destino


    SamplesRec := 0;
{    AssignFile(fGrava,sDirApp+'teste_grava.raw');
    Log1.AddLine(sDirApp+'teste_grava.raw');
    rewrite(fGrava,1);}
    for i := 1 to 2 do
    begin
      nContaThread[i] := 0;
      iSeq[i] := 0;
      iSeqAnt[i] := -1;

      vlb.SetRecordFormat(i,ffWavePCM);
      vlb.SetPlayFormat(i,ffWavePCM);
//      vlb.EnableSampleToApp(i);
//      vlb.EnableSampleToCard(i);

    end;

    chkCH1Fala.Enabled := true;
    chkCH2Fala.Enabled :=true;
    chkCH1Envia.Enabled :=true;
    chkCH2Envia.Enabled :=true;

  end
  else
  begin
    chkCH1Fala.Enabled := false;
    chkCH2Fala.Enabled := false;
    chkCH1Envia.Enabled := false;
    chkCH2Envia.Enabled := false;

    vlb.DisableSampleToApp(1);
    vlb.DisableSampleToCard(1);
    vlb.DisableSampleToCard(2);
    vlb.DisableSampleToApp(2);

    MyThread[1].terminate;
    MyThread[2].terminate;
//    CloseFile(fGrava);
  end;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 
  if vlb.DriverEnabled then
    vlb.ShutdownVoicerLib;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  sDirApp:=ExtractFilePath(Application.ExeName);


end;

procedure TForm1.chkCH1FalaClick(Sender: TObject);
begin
  vlb.EnableSampleToApp(1);
end;

procedure TForm1.chkCh1EnviaClick(Sender: TObject);
begin
  vlb.EnableSampleToCard(1);
end;

procedure TForm1.chkCH2EnviaClick(Sender: TObject);
begin
  vlb.EnableSampleToCard(2);
end;

procedure TForm1.chkCH2FalaClick(Sender: TObject);
begin
  vlb.EnableSampleToApp(2);
end;

end.
